#include <windows.h>

typedef struct {
	unsigned int l;
	unsigned int h;
} CLOCK_COUNTER,*PCLOCK_COUNTER;

double __CpuFreq;

/*********************************************************************************************
**********************************************************************************************
** RDTSC_SetCPUClock: 
**		Set CPU clock frequency. Must be used before any call to RDTSC_ComputeTimeInterval.
**********************************************************************************************
*********************************************************************************************/

#define RDTSC_SetCPUClock(Freq)	__CpuFreq = Freq

/*********************************************************************************************
**********************************************************************************************
** RDTSC_GetClock: 
**		Recover CPU clock counter. 
**********************************************************************************************
*********************************************************************************************/

#define RDTSC_GetClock(Clock)	__asm{rdtsc} __asm{mov	Clock.l,eax	} __asm{mov	Clock.h,edx}	

/*********************************************************************************************
**********************************************************************************************
** RDTSC_ComputeTimeInterval: 
**		Compute the time interval elapsed between pckStart and pckStop in seconds. 
**		Returns -1.0 if pckStop < pckStart. 
**********************************************************************************************
*********************************************************************************************/

__inline double RDTSC_ComputeTimeInterval(PCLOCK_COUNTER pckStart, PCLOCK_COUNTER pckStop)
{
	ULONGLONG ckInterval;
	ULONGLONG ckStart,ckStop;

	ckStart  = pckStart->h;
	ckStart  = ckStart << 32;
	ckStart += pckStart->l;

	ckStop  = pckStop->h;
	ckStop  = ckStop << 32;
	ckStop += pckStop->l;

	if (ckStop < ckStart){
		return (-1.0);
	}

	ckInterval = ckStop-ckStart;

	return( ((double)((LONGLONG)(ckInterval)))/__CpuFreq );

}

