/********************************************************************************************
*********************************************************************************************
** Program: RDTSCTimeExample.c
**			Application examples of RDTSCTime routines
** Programmer: G.A. Borges
** Date: 11/01/2001
*********************************************************************************************
*********************************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "RDTSCTime.h"

// Try this program with different REAL_TIME and NORMAL priority levels
#define  REAL_TIME_PRIORITY		1
//#define  REAL_TIME_PRIORITY		0

void main(void)
{
	// Variables:
	int n,tms;
	double etime;
	CLOCK_COUNTER CntInit,CntEnd;

	// Priorities:
#if REAL_TIME_PRIORITY == 1
	SetPriorityClass(GetCurrentProcess(),NORMAL_PRIORITY_CLASS);
	if(!SetProcessPriorityBoost(GetCurrentProcess(),TRUE)){
		printf("Error on SetProcessPriorityBoost");
	}
	if(!SetThreadPriorityBoost(GetCurrentThread(),TRUE)){
		printf("Error on SetThreadPriorityBoost");
	}
	if(!SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_TIME_CRITICAL)){
		printf("Error on SetThreadPriority");
	}
#endif

	// Set CPU clock
	RDTSC_SetCPUClock(450e6); // The frequency is in Hz. In this example, we use a processor running at 450 MHz 

	// ** Computing time of a printf function:
	printf("\n** Computing time of a printf function:");

	// 	Get start clock
	RDTSC_GetClock(CntInit);

	// function:
	printf("\n  Hello World!");

	// 	Get stop clock
	RDTSC_GetClock(CntEnd);

	// Compute the elapsed time in seconds:
	etime = RDTSC_ComputeTimeInterval(&CntInit,&CntEnd);

	// Print:
	printf("\nElapsed time: %f ms",1000.0*etime);

	// ** Minimum measurement time:
	printf("\n** Minimum measurement time:");

	// 	Get start clock
	RDTSC_GetClock(CntInit);
	// 	Get stop clock
	RDTSC_GetClock(CntEnd);
	// Compute the elapsed time:
	etime = RDTSC_ComputeTimeInterval(&CntInit,&CntEnd);

	// Print:
	printf("\nMinimum measurement time: %f (us)",1.0e6*etime);
  
	// ** Comparing the elapsed time of a Sleep Function for different sleeping times:
	printf("\n** Comparing the elapsed time of a Sleep Function for different sleeping times:");
	for(n=0,tms=1; n<5; ++n,tms*=10){
		RDTSC_GetClock(CntInit);
		Sleep(tms);
		RDTSC_GetClock(CntEnd);
		etime = RDTSC_ComputeTimeInterval(&CntInit,&CntEnd);
		printf("\nMeasurement time: %f (ms). Sleep time: %f (ms)",1.0e3*etime, (double)(tms));
	}

	printf("\n\nP.S. If executed on Windows, consider that this thread is running in a multitask system with a Win32 scheduler (urgh!)");

	printf("\n");
}	



